import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:zowie_flutter_sdk/zowie_color.dart';

class Zowie {
  static const MethodChannel _channel = const MethodChannel('zowie_flutter_sdk');
  static const String _cannotBeNullOrEmpty = "cannot be null or empty.";

  static Future<bool> configuration(
    String userId,
    String conversationId,
    String instanceId, {
    String conversationInitReferral,
    String jwt,
  }) async {
    assert(userId != null && userId.isNotEmpty, "userId $_cannotBeNullOrEmpty");
    assert(conversationId != null && conversationId.isNotEmpty, "conversationId $_cannotBeNullOrEmpty");
    assert(instanceId != null && instanceId.isNotEmpty, "instanceId $_cannotBeNullOrEmpty");

    final bool result = await _channel.invokeMethod(
      _methodConfiguration,
      <String, dynamic>{
        "userId": userId,
        "conversationId": conversationId,
        "instanceId": instanceId,
        "conversationInitReferral": conversationInitReferral,
        "jwt": jwt
      },
    );
    return result;
  }

  static Future<bool> showChat() async {
    final bool result = await _channel.invokeMethod(_methodShowChat);
    return result;
  }

  static Future<bool> metadata({
    String firstName,
    String lastName,
    String name,
    String locale,
    String timeZone,
    String phoneNumber,
    String email,
    Map<String, String> extraParams,
  }) async {
    final bool result = await _channel.invokeMethod(
      _methodMetadata,
      <String, dynamic>{
        "firstName": firstName,
        "lastName": lastName,
        "name": name,
        "locale": locale,
        "timeZone": timeZone,
        "phoneNumber": phoneNumber,
        "email": email,
        "extraParams": jsonEncode(extraParams),
      },
    );
    return result;
  }

  static Future<bool> enableNotifications(String fcmToken) async {
    assert(fcmToken != null && fcmToken.isNotEmpty, "fcmToken $_cannotBeNullOrEmpty");

    final bool result = await _channel.invokeMethod(
      _methodEnableNotifications,
      <String, dynamic>{"fcmToken": fcmToken},
    );
    return result;
  }

  static Future<bool> disableNotifications() async {
    final bool result = await _channel.invokeMethod(
      _methodDisableNotifications,
    );
    return result;
  }

  static Future<bool> setUserStatus(bool isActive) async {
    assert(isActive != null, "isActive cannot be null");

    final bool result = await _channel.invokeMethod(
      _methodSetUserStatus,
      <String, dynamic>{"isActive": isActive},
    );
    return result;
  }

  static Future<bool> setContext(String contextId) async {
    assert(contextId != null && contextId.isNotEmpty, "contextId $_cannotBeNullOrEmpty");

    final bool result = await _channel.invokeMethod(
      _methodSetContext,
      <String, dynamic>{"contextId": contextId},
    );
    return result;
  }

  static Future<bool> localization({
    String newMessageHint,
    String messageStatusDelivered,
    String messageStatusSendingErrorMessage,
    String messageStatusSendingErrorTryAgain,
    String readAndWriteStoragePermissionAlertTitle,
    String readAndWriteStoragePermissionAlertMessage,
    String readAndWriteStoragePermissionAlertPositiveButton,
    String readAndWriteStoragePermissionAlertNegativeButton,
    String attachmentPlaceholderName,
    String attachmentFileMaxSizeExceededErrorMessage,
    String couldNotOpenFileErrorMessage,
    String chatConnectionErrorMessage,
    String chatConnectionRestoredMessage,
    String chatHistoryDownloadErrorMessage,
    String couldNotOpenWebBrowserErrorMessage,
    String unexpectedErrorMessage,
    String fileDownloadErrorMessage,
  }) async {
    final bool result = await _channel.invokeMethod(
      _methodLocalization,
      <String, dynamic>{
        "newMessageHint": newMessageHint,
        "messageStatusDelivered": messageStatusDelivered,
        "messageStatusSendingErrorMessage": messageStatusSendingErrorMessage,
        "messageStatusSendingErrorTryAgain": messageStatusSendingErrorTryAgain,
        "readAndWriteStoragePermissionAlertTitle": readAndWriteStoragePermissionAlertTitle,
        "readAndWriteStoragePermissionAlertMessage": readAndWriteStoragePermissionAlertMessage,
        "readAndWriteStoragePermissionAlertPositiveButton": readAndWriteStoragePermissionAlertPositiveButton,
        "readAndWriteStoragePermissionAlertNegativeButton": readAndWriteStoragePermissionAlertNegativeButton,
        "attachmentPlaceholderName": attachmentPlaceholderName,
        "attachmentFileMaxSizeExceededErrorMessage": attachmentFileMaxSizeExceededErrorMessage,
        "couldNotOpenFileErrorMessage": couldNotOpenFileErrorMessage,
        "chatConnectionErrorMessage": chatConnectionErrorMessage,
        "chatConnectionRestoredMessage": chatConnectionRestoredMessage,
        "chatHistoryDownloadErrorMessage": chatHistoryDownloadErrorMessage,
        "couldNotOpenWebBrowserErrorMessage": couldNotOpenWebBrowserErrorMessage,
        "unexpectedErrorMessage": unexpectedErrorMessage,
        "fileDownloadErrorMessage": fileDownloadErrorMessage,
      },
    );
    return result;
  }

  static Future<bool> colors({
    Color messageStatusSendingErrorColor,
    Color messageStatusDeliveredColor,
    ZowieColor sentMessageBackgroundColor,
    Color sentMessageContentsColor,
    Color sentMessageImageUploadLoadingColor,
    Color sentMessageVideoUploadLoadingColor,
    Color sentMessageImagePlaceholderLoadingColor,
    Color sentMessageImagePlaceholderBackgroundColor,
    Color sentMessageLinksColor,
    ZowieColor incomingMessageBackgroundColor,
    Color incomingMessagePrimaryTextColor,
    Color incomingMessageSecondaryTextColor,
    ZowieColor incomingMessageFileIconColor,
    Color incomingMessageFileDownloadSuccessIconColor,
    Color incomingMessageDownloadFileIconColor,
    Color incomingMessageDownloadFileLoadingColor,
    Color incomingMessageImagePlaceholderLoadingColor,
    Color incomingMessageImagePlaceholderBackgroundColor,
    Color incomingMessageLinksColor,
    Color backgroundColor,
    Color newMessageTextColor,
    Color newMessageHintTextColor,
    Color sendAttachmentButtonColor,
    ZowieColor sendTextButtonColor,
    Color separatorColor,
    Color chatMessagesLoadingColor,
    Color quickButtonBackgroundColor,
    Color quickButtonTextColor,
    Color quickButtonPressedStrokeColor,
    Color actionButtonBackgroundColor,
    Color actionButtonBackgroundPressedColor,
    Color actionButtonTextColor,
    Color videoThumbnailPlaceholderColor,
    Color notificationErrorContentsColor,
    Color notificationErrorBackgroundColor,
    Color notificationSuccessContentsColor,
    Color notificationSuccessBackgroundColor,
    Color zowieLogoButtonBackgroundPressedColor,
    Color zowieLogoButtonPressedStrokeColor,
    Color playVideoButtonBackgroundColor,
    Color playVideoButtonBackgroundPressedColor,
    Color playVideoButtonPlayIconColor,
    Color actionButtonShadow,
  }) async {
    final bool result = await _channel.invokeMethod(
      _methodColors,
      <String, dynamic>{
        "messageStatusSendingErrorColor": _getColorHex(messageStatusSendingErrorColor),
        "messageStatusDeliveredColor": _getColorHex(messageStatusDeliveredColor),
        "sentMessageBackgroundColor": _getZowieColorJson(sentMessageBackgroundColor),
        "sentMessageContentsColor": _getColorHex(sentMessageContentsColor),
        "sentMessageImageUploadLoadingColor": _getColorHex(sentMessageImageUploadLoadingColor),
        "sentMessageVideoUploadLoadingColor": _getColorHex(sentMessageVideoUploadLoadingColor),
        "sentMessageImagePlaceholderLoadingColor": _getColorHex(sentMessageImagePlaceholderLoadingColor),
        "sentMessageImagePlaceholderBackgroundColor": _getColorHex(sentMessageImagePlaceholderBackgroundColor),
        "sentMessageLinksColor": _getColorHex(sentMessageLinksColor),
        "incomingMessageBackgroundColor": _getZowieColorJson(incomingMessageBackgroundColor),
        "incomingMessagePrimaryTextColor": _getColorHex(incomingMessagePrimaryTextColor),
        "incomingMessageSecondaryTextColor": _getColorHex(incomingMessageSecondaryTextColor),
        "incomingMessageFileIconColor": _getZowieColorJson(incomingMessageFileIconColor),
        "incomingMessageFileDownloadSuccessIconColor": _getColorHex(incomingMessageFileDownloadSuccessIconColor),
        "incomingMessageDownloadFileIconColor": _getColorHex(incomingMessageDownloadFileIconColor),
        "incomingMessageDownloadFileLoadingColor": _getColorHex(incomingMessageDownloadFileLoadingColor),
        "incomingMessageImagePlaceholderLoadingColor": _getColorHex(incomingMessageImagePlaceholderLoadingColor),
        "incomingMessageImagePlaceholderBackgroundColor": _getColorHex(incomingMessageImagePlaceholderBackgroundColor),
        "incomingMessageLinksColor": _getColorHex(incomingMessageLinksColor),
        "backgroundColor": _getColorHex(backgroundColor),
        "newMessageTextColor": _getColorHex(newMessageTextColor),
        "newMessageHintTextColor": _getColorHex(newMessageHintTextColor),
        "sendAttachmentButtonColor": _getColorHex(sendAttachmentButtonColor),
        "sendTextButtonColor": _getZowieColorJson(sendTextButtonColor),
        "separatorColor": _getColorHex(separatorColor),
        "chatMessagesLoadingColor": _getColorHex(chatMessagesLoadingColor),
        "quickButtonBackgroundColor": _getColorHex(quickButtonBackgroundColor),
        "quickButtonTextColor": _getColorHex(quickButtonTextColor),
        "quickButtonPressedStrokeColor": _getColorHex(quickButtonPressedStrokeColor),
        "actionButtonBackgroundColor": _getColorHex(actionButtonBackgroundColor),
        "actionButtonBackgroundPressedColor": _getColorHex(actionButtonBackgroundPressedColor),
        "actionButtonTextColor": _getColorHex(actionButtonTextColor),
        "videoThumbnailPlaceholderColor": _getColorHex(videoThumbnailPlaceholderColor),
        "notificationErrorContentsColor": _getColorHex(notificationErrorContentsColor),
        "notificationErrorBackgroundColor": _getColorHex(notificationErrorBackgroundColor),
        "notificationSuccessContentsColor": _getColorHex(notificationSuccessContentsColor),
        "notificationSuccessBackgroundColor": _getColorHex(notificationSuccessBackgroundColor),
        "zowieLogoButtonBackgroundPressedColor": _getColorHex(zowieLogoButtonBackgroundPressedColor),
        "zowieLogoButtonPressedStrokeColor": _getColorHex(zowieLogoButtonPressedStrokeColor),
        "playVideoButtonBackgroundColor": _getColorHex(playVideoButtonBackgroundColor),
        "playVideoButtonBackgroundPressedColor": _getColorHex(playVideoButtonBackgroundPressedColor),
        "playVideoButtonPlayIconColor": _getColorHex(playVideoButtonPlayIconColor),
        "actionButtonShadow": _getColorHex(actionButtonShadow),
      },
    );
    return result;
  }

  static String _getColorHex(Color color) {
    if (color == null) {
      return null;
    } else {
      return "#${color.value.toRadixString(16)}";
    }
  }

  static String _getZowieColorJson(ZowieColor color) {
    if (color == null) {
      return null;
    } else if (color is SolidColor) {
      return jsonEncode(<String, String>{
        "type": "solid",
        "color": _getColorHex(color.color),
      });
    } else if (color is GradientColors) {
      return jsonEncode(<String, String>{
        "type": "gradient",
        "startColor": _getColorHex(color.startColor),
        "endColor": _getColorHex(color.endColor),
      });
    } else {
      return null;
    }
  }

  static String _methodConfiguration = "configuration";
  static String _methodShowChat = "showChat";
  static String _methodMetadata = "metadata";
  static String _methodEnableNotifications = "enableNotifications";
  static String _methodDisableNotifications = "disableNotifications";
  static String _methodSetUserStatus = "setUserStatus";
  static String _methodSetContext = "setContext";
  static String _methodLocalization = "localization";
  static String _methodColors = "colors";
}
